<?php

namespace Drupal\access_filter\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\access_filter\Entity\Filter;
use Symfony\Component\Yaml\Dumper;
use Symfony\Component\Yaml\Parser;

/**
 * Provides edit filter form.
 */
class EditForm extends EntityForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    /* @var Filter $filter */
    $filter = $this->entity;
    $filter->parse();

    $form['general'] = array(
      '#type' => 'fieldset',
      '#title' => t('General'),
    );
    $form['general']['status'] = array(
      '#type' => 'checkbox',
      '#title' => t('Enabled'),
      '#default_value' => $filter->status(),
    );
    $form['general']['name'] = array(
      '#type' => 'textfield',
      '#title' => t('Name'),
      '#default_value' => $filter->label(),
      '#max_length' => 255,
    );
    $form['general']['id'] = array(
      '#type' => 'machine_name',
      '#default_value' => $filter->id(),
      '#machine_name' => array(
        'source' => array('general', 'name'),
        'exists' => array(Filter::class, 'load'),
      ),
      '#disabled' => !$filter->isNew(),
    );

    $form['conditions'] = array(
      '#type' => 'fieldset',
      '#title' => t('Conditions'),
    );
    $form['conditions']['conditions'] = array(
      '#type' => 'textarea',
      '#title' => $this->t('Conditions'),
      '#title_display' => 'invisible',
      '#description' => nl2br($this->t("Write in YAML format.\n@example", array(
        '@example' => '- { type: foo, param1: bar, ... }',
      ))),
      '#default_value' => $filter->conditions,
    );

    /* @var \Drupal\access_filter\Plugin\AccessFilterPluginManager $condition_plugin_manager */
    $condition_plugin_manager = \Drupal::service('plugin.manager.access_filter.condition');
    $condition_plugins = array();
    foreach ($condition_plugin_manager->getDefinitions() as $id => $definition) {
      $condition_plugins[] = $id . ': ' . $definition['description'];
    }
    $form['conditions']['plugins'] = array(
      '#theme' => 'item_list',
      '#items' => $condition_plugins,
      '#prefix' => '<div class="description">' . $this->t('Types') . '</div>',
    );

    $form['rules'] = array(
      '#type' => 'fieldset',
      '#title' => t('Rules'),
    );
    $form['rules']['rules'] = array(
      '#type' => 'textarea',
      '#title' => $this->t('Rules'),
      '#title_display' => 'invisible',
      '#description' => nl2br($this->t("Write in YAML format.\n@example", array(
        '@example' => '- { type: foo, action: deny, param1: bar, ... }',
      ))),
      '#default_value' => $filter->rules,
    );

    /* @var \Drupal\access_filter\Plugin\AccessFilterPluginManager $ruleplugin_manager */
    $ruleplugin_manager = \Drupal::service('plugin.manager.access_filter.rule');
    $rule_plugins = array();
    foreach ($ruleplugin_manager->getDefinitions() as $id => $definition) {
      $rule_plugins[] = $id . ': ' . $definition['description'];
    }
    $form['rules']['plugins'] = array(
      '#theme' => 'item_list',
      '#items' => $rule_plugins,
      '#prefix' => '<div class="description">' . $this->t('Types') . '</div>',
    );

    $form['response'] = array(
      '#type' => 'fieldset',
      '#title' => t('Response'),
      '#tree' => TRUE,
    );
    $form['response']['code'] = array(
      '#type' => 'select',
      '#title' => t('Response code'),
      '#options' => array(
        403 => '403 Forbidden',
        404 => '404 Not Found',
        301 => '301 Moved Permanently',
        302 => '302 Moved Temporarily',
        200 => '200 OK',
      ),
      '#default_value' => $filter->parsedResponse['code'],
    );
    $form['response']['redirect_url'] = array(
      '#type' => 'textfield',
      '#title' => t('Redirect URL'),
      '#description' => t('Affects only response code 301, 302.'),
      '#default_value' => $filter->parsedResponse['redirect_url'],
    );
    $form['response']['body'] = array(
      '#type' => 'textarea',
      '#title' => t('Response body'),
      '#description' => t('Affects only response code except 301, 302.'),
      '#default_value' => $filter->parsedResponse['body'],
    );

    $form['actions']['save'] = array(
      '#type' => 'submit',
      '#value' => $this->t('Save'),
    );

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $parser = new Parser();

    // Validate configuration for conditions.
    $conditions = array();
    try {
      $conditions = $parser->parse($form_state->getValue('conditions'));
    }
    catch (\Exception $ex) {
      $form_state->setErrorByName('conditions', $this->t('Invalid YAML format for conditions. @exception', array('@exception' => $ex->getMessage())));
    }

    /* @var \Drupal\access_filter\Plugin\AccessFilterPluginManager $condition_plugin_manager */
    $condition_plugin_manager = \Drupal::service('plugin.manager.access_filter.condition');
    $condition_plugins = $condition_plugin_manager->getDefinitions();
    $condition_errors = array();
    foreach ($conditions as $i => $condition) {
      $line_errors = array();

      if (isset($condition['type']) && strlen($condition['type'])) {
        $plugin_id = $condition['type'];
        if (isset($condition_plugins[$plugin_id])) {
          $instance = $condition_plugin_manager->createInstance($plugin_id, $condition);
          $line_errors += array_values($instance->validateConfiguration($condition));
        }
        else {
          $line_errors[] = $this->t('Type @type is invalid.', array(
            '@type' => $plugin_id,
          ));
        }
      }
      else {
        $line_errors[] = $this->t("'@property' is required.", array('@property' => 'type'));
      }

      if (!empty($line_errors)) {
        $condition_errors[$i] = $line_errors;
      }
    }

    if (!empty($condition_errors)) {
      $form_state->setErrorByName('conditions', '');
      drupal_set_message($this->t('There are configuration errors in conditions.'), 'error', TRUE);

      foreach ($condition_errors as $i => $line_errors) {
        drupal_set_message($this->t('Line @line:', array('@line' => $i + 1)), 'error', TRUE);

        foreach ($line_errors as $error) {
          drupal_set_message('- ' . $error, 'error', TRUE);
        }
      }
    }

    // Validate configuration for rules.
    $rules = array();
    try {
      $rules = $parser->parse($form_state->getValue('rules'));
    }
    catch (\Exception $ex) {
      $form_state->setErrorByName('rules', $this->t('Invalid YAML format for rules. @exception', array('@exception' => $ex->getMessage())));
    }

    /* @var \Drupal\access_filter\Plugin\AccessFilterPluginManager $rule_plugin_manager */
    $rule_plugin_manager = \Drupal::service('plugin.manager.access_filter.rule');
    $rule_plugins = $rule_plugin_manager->getDefinitions();
    $rule_errors = array();
    foreach ($rules as $i => $rule) {
      $line_errors = array();

      if (isset($rule['type']) && strlen($rule['type'])) {
        $plugin_id = $rule['type'];
        if (isset($rule_plugins[$plugin_id])) {
          $instance = $rule_plugin_manager->createInstance($plugin_id, $rule);

          if (!isset($rule['action']) || !strlen($rule['action'])) {
            $line_errors[] = $this->t("'@property' is required.", array('@property' => 'action'));
          }
          elseif (!in_array($rule['action'], array('deny', 'allow'))) {
            $line_errors[] = $this->t("'action' should be 'deny' or 'allow'.");
          }
          $line_errors += array_values($instance->validateConfiguration($rule));
        }
        else {
          $line_errors[] = $this->t('Type @type is invalid.', array(
            '@type' => $plugin_id,
          ));
        }
      }
      else {
        $line_errors[] = $this->t("'@property' is required.", array('@property' => 'type'));
      }

      if (!empty($line_errors)) {
        $rule_errors[$i] = $line_errors;
      }
    }

    if (!empty($rule_errors)) {
      $form_state->setErrorByName('rules', '');
      drupal_set_message($this->t('There are configuration errors in rules.'), 'error', TRUE);

      foreach ($rule_errors as $i => $line_errors) {
        drupal_set_message($this->t('Line @line:', array('@line' => $i + 1)), 'error', TRUE);

        foreach ($line_errors as $error) {
          drupal_set_message('- ' . $error, 'error', TRUE);
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $filter = $this->entity;

    $dumper = new Dumper();
    $filter->response = $dumper->dump($form_state->getValue('response'), FALSE);

    $status = $filter->save();
    if ($status) {
      drupal_set_message($this->t('Filter %label has been saved.', array(
        '%label' => $filter->name,
      )));
    }
    else {
      drupal_set_message($this->t('Filter %label was not saved.', array(
        '%label' => $filter->name,
      )));
    }

    $form_state->setRedirect('entity.access_filter.collection');
  }

}
