<?php

namespace Drupal\access_filter\Form;

use Drupal\Component\Utility\Html;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Site\Settings;
use Drupal\Core\Url;
use Drupal\access_filter\Entity\Filter;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides filter overview form.
 */
class Overview extends FormBase {

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static();
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'access_filter_overview';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    if (Settings::get('access_filter_disabled')) {
      drupal_set_message($this->t('Access control is currently disabled in settings.php.'), 'warning');
    }

    $form['filters'] = array(
      '#type' => 'table',
      '#attributes' => array('id' => 'filters'),
      '#header' => array(
        $this->t('ID'),
        $this->t('Name'),
        $this->t('Conditions'),
        $this->t('Rules'),
        $this->t('Response'),
        $this->t('Weight'),
        $this->t('Operations'),
      ),
      '#empty' => $this->t('No filters available.'),
    );

    $destination = $this->getDestinationArray();

    $filters = Filter::loadMultiple();
    $form_state->set('filters', $filters);

    foreach ($filters as $filter) {
      $filter->parse();

      $row = array(
        '#attributes' => array('class' => array('draggable')),
      );

      $row['id'] = array(
        '#markup' => $filter->id,
      );
      $row['name'] = array(
        '#markup' => Html::escape($filter->name),
      );

      $conditions = array();
      foreach ($filter->parsedConditions as $condition) {
        $conditions[] = $this->t('@type: @summary', array(
          '@type' => $condition->getPluginId(),
          '@summary' => $condition->summary(),
        ));
      }
      $row['conditions'] = array(
        '#theme' => 'item_list',
        '#items' => $conditions,
      );

      $rules = array();
      foreach ($filter->parsedRules as $rule) {
        $rules[] = $this->t('@type: @summary', array(
          '@type' => $rule->getPluginId(),
          '@summary' => $rule->summary(),
        ));
      }
      $row['rules'] = array(
        '#theme' => 'item_list',
        '#items' => $rules,
      );

      $row['response'] = array(
        '#markup' => Html::escape($filter->parsedResponse['code']),
      );

      $operations = array();
      $operations['edit'] = array(
        'title' => $this->t('Edit'),
        'query' => $destination,
        'url' => Url::fromRoute('entity.access_filter.edit_form', array('access_filter' => $filter->id)),
      );
      $operations['delete'] = array(
        'title' => $this->t('Delete'),
        'query' => $destination,
        'url' => Url::fromRoute('entity.access_filter.delete_form', array('access_filter' => $filter->id)),
      );

      $row['weight'] = array(
        '#type' => 'weight',
        '#title' => $this->t('Weight for filter.'),
        '#title_display' => 'invisible',
        '#delta' => 20,
        '#attributes' => array(
          'class' => array('filter-weight'),
        ),
        '#default_value' => $filter->weight,
      );

      $row['operations'] = array(
        '#type' => 'operations',
        '#links' => $operations,
      );

      $form['filters'][$filter->id] = $row;
    }

    $form['filters']['#tabledrag'][] = array(
      'action' => 'order',
      'relationship' => 'sibling',
      'group' => 'filter-weight',
    );

    $form['actions']['save'] = array(
      '#type' => 'submit',
      '#value' => $this->t('Save'),
    );

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    /* @var Filter[] $filters */
    $filters = $form_state->get('filters');
    /* @var Filter[] $changed_filters */
    $changed_filters = array();
    $values = $form_state->getValue('filters');
    foreach ($values as $fid => $value) {
      if ($filters[$fid]->weight != $value) {
        $filters[$fid]->weight = $value['weight'];
        $changed_filters[] = $filters[$fid];
      }
    }

    foreach ($changed_filters as $filter) {
      $filter->save();
    }
  }

}
